# Turta Sensor uHAT Helper for Raspberry Pi OS.
# Distributed under the terms of the MIT license.

# Python Library for Bosch Sensortec BME280 Environmental Sensor.
# Version 1.0.0
# Released: June 14th, 2020

# Visit https://docs.turta.io for documentation.

import time
import math
from enum import IntEnum
from smbus import SMBus

#Enumerations

class HumidityOversampling(IntEnum):
    Skipped = 0b00000000
    x01 = 0b00000001
    x02 = 0b00000010
    x04 = 0b00000011
    x08 = 0b00000100
    x16 = 0b00000101

class TemperatureOversampling(IntEnum):
    Skipped = 0b00000000
    x01 = 0b00100000
    x02 = 0b01000000
    x04 = 0b01100000
    x08 = 0b10000000
    x16 = 0b10100000

class PressureOversampling(IntEnum):
    Skipped = 0b00000000
    x01 = 0b00000100
    x02 = 0b00001000
    x04 = 0b00001100
    x08 = 0b00010000
    x16 = 0b00010100

class SensorMode(IntEnum):
    Sleep = 0b00000000
    Forced = 0b00000001
    Normal = 0b00000011

class InactiveDuration(IntEnum):
    ms0000_5 = 0b00000000
    ms0062_5 = 0b00100000
    ms0125 = 0b01000000
    ms0250 = 0b01100000
    ms0500 = 0b10000000
    ms1000 = 0b10100000
    ms0010 = 0b11000000
    ms0020 = 0b11100000

class FilterCoefficient(IntEnum):
    FilterOff = 0b00000000
    fc02 = 0b00000100
    fc04 = 0b00001000
    fc08 = 0b00001100
    fc16 = 0b00010000

class EnvironmentalSensor(object):
    """BME280 Environmental Sensor"""

    #I2C Slave Address
    I2C_ADDRESS = 0x77

    #Registers
    BME280_SIGNATURE = 0x60
    BME280_ID = 0xD0
    BME280_RESET = 0xE0
    BME280_CTRL_HUM = 0xF2
    BME280_STATUS = 0xF3
    BME280_CTRL_MEAS = 0xF4
    BME280_CONFIG = 0xF5

    #Registers: Readout
    BME280_PRESS_MSB = 0xF7
    BME280_PRESS_LSB = 0xF8
    BME280_PRESS_XLSB = 0xF9
    BME280_TEMP_MSB = 0xFA
    BME280_TEMP_LSB = 0xFB
    BME280_TEMP_XLSB = 0xFC
    BME280_HUM_MSB = 0xFD
    BME280_HUM_LSB = 0xFE

    #Registers: Calibration
    BME280_DIG_T1 = 0x88
    BME280_DIG_T2 = 0x8A
    BME280_DIG_T3 = 0x8C
    BME280_DIG_P1 = 0x8E
    BME280_DIG_P2 = 0x90
    BME280_DIG_P3 = 0x92
    BME280_DIG_P4 = 0x94
    BME280_DIG_P5 = 0x96
    BME280_DIG_P6 = 0x98
    BME280_DIG_P7 = 0x9A
    BME280_DIG_P8 = 0x9C
    BME280_DIG_P9 = 0x9E
    BME280_DIG_H1 = 0xA1
    BME280_DIG_H2 = 0xE1
    BME280_DIG_H3 = 0xE3
    BME280_DIG_H4 = 0xE4
    BME280_DIG_H5 = 0xE5
    BME280_DIG_H6 = 0xE7

    #Data: Calibration
    calDig_T1 = None
    calDig_T2 = None
    calDig_T3 = None
    calDig_P1 = None
    calDig_P2 = None
    calDig_P3 = None
    calDig_P4 = None
    calDig_P5 = None
    calDig_P6 = None
    calDig_P7 = None
    calDig_P8 = None
    calDig_P9 = None
    calDig_H1 = None
    calDig_H2 = None
    calDig_H3 = None
    calDig_H4 = None
    calDig_H5 = None
    calDig_H6 = None
    fineTemperature = 0

    #I2C Config
    bus = SMBus(1)

    #I2C Communication

    def _write_register(self, reg_addr, data):
        """Writes data to the I2C device.

        Parameters:
        reg_addr (byte): Register address
        data (byte): Data"""

        self.bus.write_i2c_block_data(self.I2C_ADDRESS, reg_addr, [ data & 0xFF ])

    def _read_register_1ubyte(self, reg_addr):
        """Reads data from the I2C device.

        Parameters:
        reg_addr (byte): Read register address
        
        Returns:
        ubyte: Response from the device"""

        buffer = self.bus.read_i2c_block_data(self.I2C_ADDRESS, reg_addr, 1)
        return buffer[0]

    def _read_register_1sbyte(self, reg_addr):
        """Reads data from the I2C device.

        Parameters:
        reg_addr (byte): Read register address
        
        Returns:
        byte: Response from the device"""

        buffer = self.bus.read_i2c_block_data(self.I2C_ADDRESS, reg_addr, 1)
        val = buffer[0]
        if val & (1 << 7) != 0:
            val = val - (1 << 7)
        return val

    def _read_2bytes_as_ushort_lsbfirst(self, reg_addr):
        """Reads data from the I2C device.

        Parameters:
        reg_addr (byte): Read register address
        
        Returns:
        ushort: Response from the device"""

        buffer = self.bus.read_i2c_block_data(self.I2C_ADDRESS, reg_addr, 2)
        return buffer[0] + (buffer[1] << 8)

    def _read_2bytes_as_short_lsbfirst(self, reg_addr):
        """Reads data from the I2C device.

        Parameters:
        reg_addr (byte): Read register address
        
        Returns:
        short: Response from the device"""

        buffer = self.bus.read_i2c_block_data(self.I2C_ADDRESS, reg_addr, 2)
        val = buffer[0] + (buffer[1] << 8)
        if val & (1 << 15) != 0:
            val = val - (1 << 15)
        return val

    def _read_multiple_bytes_as_array(self, reg_addr, length):
        """Reads data from the I2C device.

        Parameters:
        reg_addr (byte): Read register address.
        length (byte): Data length
        
        Returns:
        byte[]: Response from the device"""

        return self.bus.read_i2c_block_data(self.I2C_ADDRESS, reg_addr, length)

    def __init__(self):
        """Initiates the BME280 sensor to get temperature, humidity, pressure and altitude."""

        self._read_calibration_data()
        self.set_oversamplings_and_mode(
            HumidityOversampling.x08,
            TemperatureOversampling.x08,
            PressureOversampling.x16,
            SensorMode.Normal)
        self.set_config(
            InactiveDuration.ms1000,
            FilterCoefficient.fc04)

    #Sensor Configuration

    def set_oversamplings_and_mode(self, humidity_oversampling, temperature_oversampling, pressure_oversamling, mode):
        """Sets the oversamplings and sensor mode.

        Parameters:
        humidity_oversampling (HumidityOversampling): Humidity oversampling.
        temperature_oversampling (TemperatureOversampling): Temperature Oversampling.
        pressure_oversamling (PressureOversampling): Pressure oversampling.
        mode (SensorMode): Sensor mode."""

        self._write_register(self.BME280_CTRL_HUM, humidity_oversampling)

        time.sleep(0.001)

        self._write_register(self.BME280_CTRL_MEAS, temperature_oversampling | pressure_oversamling | mode)

    def set_config(self, inactive_duration, filter_coefficient):
        """Sets the sensor configuration.

        Parameters:
        inactive_duration (InactiveDuration): Inactive duration between normal mode measurements.
        filter_coefficient (FilterCoefficient): Filter coefficient."""
        
        self._write_register(self.BME280_CONFIG, inactive_duration | filter_coefficient)

    #Calibration and Compensation

    def _read_calibration_data(self):
        """Reads the factory out calibration data from the sensor."""

        #Declare global variables.
        global calDig_T1
        global calDig_T2
        global calDig_T3
        global calDig_P1
        global calDig_P2
        global calDig_P3
        global calDig_P4
        global calDig_P5
        global calDig_P6
        global calDig_P7
        global calDig_P8
        global calDig_P9
        global calDig_H1
        global calDig_H2
        global calDig_H3
        global calDig_H4
        global calDig_H5
        global calDig_H6

        #Temperature calibration
        calDig_T1 = self._read_2bytes_as_ushort_lsbfirst(self.BME280_DIG_T1)
        calDig_T2 = self._read_2bytes_as_short_lsbfirst(self.BME280_DIG_T2)
        calDig_T3 = self._read_2bytes_as_short_lsbfirst(self.BME280_DIG_T3)

        #Pressure calibration
        calDig_P1 = self._read_2bytes_as_ushort_lsbfirst(self.BME280_DIG_P1)
        calDig_P2 = self._read_2bytes_as_short_lsbfirst(self.BME280_DIG_P2)
        calDig_P3 = self._read_2bytes_as_short_lsbfirst(self.BME280_DIG_P3)
        calDig_P4 = self._read_2bytes_as_short_lsbfirst(self.BME280_DIG_P4)
        calDig_P5 = self._read_2bytes_as_short_lsbfirst(self.BME280_DIG_P5)
        calDig_P6 = self._read_2bytes_as_short_lsbfirst(self.BME280_DIG_P6)
        calDig_P7 = self._read_2bytes_as_short_lsbfirst(self.BME280_DIG_P7)
        calDig_P8 = self._read_2bytes_as_short_lsbfirst(self.BME280_DIG_P8)
        calDig_P9 = self._read_2bytes_as_short_lsbfirst(self.BME280_DIG_P9)

        #Humidity calibration
        calDig_H1 = self._read_register_1sbyte(self.BME280_DIG_H1)
        calDig_H2 = self._read_2bytes_as_ushort_lsbfirst(self.BME280_DIG_H2)
        calDig_H3 = self._read_register_1sbyte(self.BME280_DIG_H3)
        calDig_H4 = (self._read_register_1sbyte(self.BME280_DIG_H4) << 4) | (self._read_register_1sbyte(self.BME280_DIG_H4 + 1) & 0xF)
        calDig_H5 = self._read_register_1sbyte((self.BME280_DIG_H5 + 1) << 4) | (self._read_register_1sbyte(self.BME280_DIG_H5) >> 4)
        calDig_H6 = self._read_register_1sbyte(self.BME280_DIG_H6)

    def _compensate_temperature(self, temp_adc):
        """Compensates the temperature.

        Parameters:
        temp_adc (uint): Analog temperature value

        Returns:
        Float: Temperature"""

        global fineTemperature

        var1 = (temp_adc / 16384.0 - calDig_T1 / 1024.0) * calDig_T2
        var2 = ((temp_adc / 131072.0 - calDig_T1 / 8192.0) * (temp_adc / 131072.0 - calDig_T1 / 8192.0)) * calDig_T3
        fineTemperature = (var1 + var2)
        val = fineTemperature / 5120.0

        return float(val)

    def _compensate_pressure(self, pres_adc):
        """Compensates the pressure.

        Parameters:
        pres_adc (uint): Analog pressure value

        Returns:
        Float: Pressure"""

        var1 = (fineTemperature / 2.0) - 64000.0
        var2 = var1 * var1 * calDig_P6 / 32768.0
        var2 = var2 + var1 * calDig_P5 * 2.0
        var2 = (var2 / 4.0) + (calDig_P4 * 65536.0)
        var1 = (calDig_P3 * var1 * var1 / 524288.0 + calDig_P2 * var1) / 524288.0
        var1 = (1.0 + var1 / 32768.0) * calDig_P1

        if var1 == 0.0:
            return 0

        val = 1048576.0 - pres_adc
        val = (val - (var2 / 4096.0)) * 6250.0 / var1
        var1 = calDig_P9 * val * val / 2147483648.0
        var2 = val * calDig_P8 / 32768.0
        val = val + (var1 + var2 + calDig_P7) / 16.0

        return float(val)

    def _compensate_humidity(self, hum_adc):
        """Compensates the humidity.

        Parameters:
        hum_adc (uint): Analog humidity value

        Returns:
        Float: Relative Humidity"""

        val = fineTemperature - 76800.0
        val = (hum_adc - (calDig_H4 * 64.0 + calDig_H5 / 16384.0 * val)) * (calDig_H2 / 65536.0 * (1.0 + calDig_H6 / 67108864.0 * val * (1.0 + calDig_H3 / 67108864.0 * val)))
        val = val * (1.0 - calDig_H1 * val / 524288.0)

        if val > 100.0:
            val = 100.0
        elif val < 0.0:
            val = 0.0

        return val

    #Sensor Readouts

    def read_temperature(self):
        """Reads the temperature in Celcius.
        
        Returns:
        Float: Temperature"""

        tRaw = self._read_multiple_bytes_as_array(self.BME280_TEMP_MSB, 3)

        return float(self._compensate_temperature((tRaw[0] << 12) + (tRaw[1] << 4) + (tRaw[2] >> 4)))

    def read_humidity(self):
        """Reads the relative humidity.
        
        Returns:
        Float: Relative Humidity"""

        hRaw = self._read_multiple_bytes_as_array(self.BME280_HUM_MSB, 2)

        return float(self._compensate_humidity((hRaw[0] << 8) + hRaw[1]))

    def read_pressure(self):
        """Reads the pressure in Pa.
        
        Returns:
        Float: Pressure"""

        pRaw = self._read_multiple_bytes_as_array(self.BME280_PRESS_MSB, 3)

        return float(self._compensate_pressure((pRaw[0] << 12) + (pRaw[1] << 4) + (pRaw[2] >> 4)))

    def read_altitude(self, meanSeaLevelPressureInBar):
        """Reads the altitude from the sea level in meters.

        Parameters:
        meanSeaLevelPressureInBar: Mean sea level pressure in bar. Will be used for altitude calculation from the pressure.
        
        Returns:
        Float: Altitude"""

        phPa = self.read_pressure() / 100.0

        return float(44330.0 * (1.0 - math.pow((phPa / meanSeaLevelPressureInBar), 0.1903)))

    def read_tph(self):
        """Reads temperature, pressure and relative humidity.
        
        Returns:
        Float[]: Temperatıre, Pressure, Relative Humidity values respectively"""

        resultsTPH = [ 0.0, 0.0, 0.0 ]

        tRaw = self._read_multiple_bytes_as_array(self.BME280_TEMP_MSB, 3)
        pRaw = self._read_multiple_bytes_as_array(self.BME280_PRESS_MSB, 3)
        hRaw = self._read_multiple_bytes_as_array(self.BME280_HUM_MSB, 2)

        resultsTPH[0] = float(self._compensate_temperature((tRaw[0] << 12) + (tRaw[1] << 4) + (tRaw[2] >> 4)))
        resultsTPH[1] = float(self._compensate_pressure((pRaw[0] << 12) + (pRaw[1] << 4) + (pRaw[2] >> 4)))
        resultsTPH[2] = float(self._compensate_humidity(hRaw[0] << 8) + hRaw[1])

        return resultsTPH

    #Disposal
    def __del__(self):
        """Releases the resources."""

        #Turn off the sensor.
        self.set_oversamplings_and_mode(
            HumidityOversampling.Skipped,
            TemperatureOversampling.Skipped,
            PressureOversampling.Skipped,
            SensorMode.Sleep)

        global calDig_T1
        global calDig_T2
        global calDig_T3
        global calDig_P1
        global calDig_P2
        global calDig_P3
        global calDig_P4
        global calDig_P5
        global calDig_P6
        global calDig_P7
        global calDig_P8
        global calDig_P9
        global calDig_H1
        global calDig_H2
        global calDig_H3
        global calDig_H4
        global calDig_H5
        global calDig_H6
        global fineTemperature

        del calDig_T1
        del calDig_T2
        del calDig_T3
        del calDig_P1
        del calDig_P2
        del calDig_P3
        del calDig_P4
        del calDig_P5
        del calDig_P6
        del calDig_P7
        del calDig_P8
        del calDig_P9
        del calDig_H1
        del calDig_H2
        del calDig_H3
        del calDig_H4
        del calDig_H5
        del calDig_H6
        del fineTemperature



# Turta Sensor uHAT Helper for Raspberry Pi OS.
# Distributed under the terms of the MIT license.

# Python Library for Digital IO Ports.
# Version 1.0.0
# Released: June 14th, 2020

# Visit https://docs.turta.io for documentation.

import RPi.GPIO as GPIO

class DigitalPort(object):
    """Digital IO Ports."""

    #Variables
    is_initialized = False

    #Port Pins
    d1, d2, d3, d4 = 21, 22, 23, 24

    def __init__(self, d1In = True, d2In = True, d3In = True, d4In = True):
        """Initiates the GPIO pins.

        Parameters:
        d1In (bool): Pin 1 direction (True for input, False for output, True is default)
        d2In (bool): Pin 2 direction (True for input, False for output, True is default)
        d3In (bool): Pin 3 direction (True for input, False for output, True is default)
        d4In (bool): Pin 4 direction (True for input, False for output, True is default)"""

        GPIO.setwarnings(False)
        GPIO.setmode(GPIO.BCM)

        if (d1In):
            GPIO.setup(self.d1, GPIO.IN, pull_up_down = GPIO.PUD_DOWN)
        else:
            GPIO.setup(self.d1, GPIO.OUT)
            GPIO.output(self.d1, GPIO.LOW)

        if (d2In):
            GPIO.setup(self.d2, GPIO.IN, pull_up_down = GPIO.PUD_DOWN)
        else:
            GPIO.setup(self.d2, GPIO.OUT)
            GPIO.output(self.d2, GPIO.LOW)

        if (d3In):
            GPIO.setup(self.d3, GPIO.IN, pull_up_down = GPIO.PUD_DOWN)
        else:
            GPIO.setup(self.d3, GPIO.OUT)
            GPIO.output(self.d3, GPIO.LOW)

        if (d4In):
            GPIO.setup(self.d4, GPIO.IN, pull_up_down = GPIO.PUD_DOWN)
        else:
            GPIO.setup(self.d4, GPIO.OUT)
            GPIO.output(self.d4, GPIO.LOW)

        self.is_initialized = True
        return

    #GPIO Read and Write Methods

    def read(self, ch):
        """Reads the digital input.

        Parameters:
        ch (byte): IO Channel (1 to 4)

        Returns:
        bool: Pin input state (True for high, False for low)"""

        if (ch == 1):
            return GPIO.input(self.d1)
        elif (ch == 2):
            return GPIO.input(self.d2)
        elif (ch == 3):
            return GPIO.input(self.d3)
        elif (ch == 4):
            return GPIO.input(self.d4)
        else:
            return 0

    def write(self, ch, st):
        """Sets the digital output.

        Parameters:
        ch (byte): IO Channel (1 to 4)
        st (bool): Pin output state (True for high, False for low)"""

        if (ch == 1):
            GPIO.output(self.d1, GPIO.HIGH if st else GPIO.LOW)
        elif (ch == 2):
            GPIO.output(self.d2, GPIO.HIGH if st else GPIO.LOW)
        elif (ch == 3):
            GPIO.output(self.d3, GPIO.HIGH if st else GPIO.LOW)
        elif (ch == 4):
            GPIO.output(self.d4, GPIO.HIGH if st else GPIO.LOW)
        return

    def toggle(self, ch):
        """Inverts the digital output.

        Parameters:
        ch (byte): IO Channel (1 to 4)"""

        self.write(ch, not self.read(ch))
        return

    #Disposal

    def __del__(self):
        """Releases the resources."""

        try:
            if self.is_initialized:
                GPIO.cleanup()
                del self.is_initialized
        except:
            pass



# Turta Sensor uHAT Helper for Raspberry Pi OS.
# Distributed under the terms of the MIT license.

# Python Library for Silicon Labs Si1133 UV Index & Ambient Light Sensor.
# Version 1.0.0
# Released: June 14th, 2020

# Visit https://docs.turta.io for documentation.

from enum import IntEnum
import RPi.GPIO as GPIO
from smbus import SMBus
from time import sleep

#Enumerations

#MODES: Operational Modes
class MODES(IntEnum):
    OFF = 0
    INITIALIZATION = 1
    STAND_BY = 2
    FORCED_CONVERSATION = 3
    AUTONOMOUS = 4

class UVAmbientLightSensor:
    """Si1133 UV Index & Ambient Light Sensor"""

    #I2C Slave Address
    I2C_ADDRESS = 0x52

    #Registers
    PART_ID = 0x00
    HW_ID = 0x01
    REV_ID = 0x02
    INFO0 = 0x03
    INFO1 = 0x04
    HOSTIN0 = 0x0A
    HOSTIN1 = 0x09
    HOSTIN2 = 0x08
    HOSTIN3 = 0x07
    COMMAND = 0x0B
    RESET = 0x0F
    RESPONSE1 = 0x10
    RESPONSE0 = 0x11
    IRQ_STATUS = 0x12
    HOSTOUT0 = 0x13
    HOSTOUT1 = 0x14
    HOSTOUT2 = 0x15
    HOSTOUT3 = 0x16
    HOSTOUT4 = 0x17
    HOSTOUT5 = 0x18
    HOSTOUT6 = 0x19
    HOSTOUT7 = 0x1A
    HOSTOUT8 = 0x1B
    HOSTOUT9 = 0x1C
    HOSTOUT10 = 0x1D
    HOSTOUT11 = 0x1E
    HOSTOUT12 = 0x1F
    HOSTOUT13 = 0x20
    HOSTOUT14 = 0x21
    HOSTOUT15 = 0x22
    HOSTOUT16 = 0x23
    HOSTOUT17 = 0x24
    HOSTOUT18 = 0x25
    HOSTOUT19 = 0x26
    HOSTOUT20 = 0x27
    HOSTOUT21 = 0x28
    HOSTOUT22 = 0x29
    HOSTOUT23 = 0x2A
    HOSTOUT24 = 0x2B
    HOSTOUT25 = 0x2C

    #Parameters
    I2CADDR = 0X00
    CHLIST = 0X01
    ADCCONFIG0 = 0X02
    ADCSENS0 = 0X03
    ADCPSOT0 = 0x04
    MEASCONFIG0 = 0x05
    ADCCONFIG1 = 0x06
    ADCSENS1 = 0x07
    ADCPSOT1 = 0x08
    MEASCONFIG1 = 0x09
    ADCCONFIG20 = 0x0A
    ADCSENS2 = 0x0B
    ADCPSOT2 = 0x0C
    MEASCONFIG2 = 0x0D
    ADCCONFIG3 = 0x0E
    ADCSENS3 = 0x0F
    ADCPSOT3 = 0x10
    MEASCONFIG3 = 0x11
    ADCCONFIG4 = 0x12
    ADCSENS4 = 0x13
    ADCPSOT4 = 0x14
    MEASCONFIG4 = 0x15
    ADCCONFIG5 = 0x16
    ADCSENS5 = 0x17
    ADCPSOT5 = 0x18
    MEASCONFIG5 = 0x19
    MEASRATEH = 0x1A
    MEASRATEL = 0x1B
    MEASCOUNT0 = 0x1C
    MEASCOUNT1 = 0x1D
    MEASCOUNT2 = 0x1E
    THRESHOLD0_H = 0x25
    THRESHOLD0_L = 0x26  
    THRESHOLD1_H = 0x27
    THRESHOLD1_L = 0x28
    THRESHOLD2_H = 0x29
    THRESHOLD2_L = 0x2A
    BURST = 0x2B

    #Commands
    RESET_CMD_CTR = 0x00
    RESET_SW = 0x01
    FORCE = 0x11
    PAUSE = 0x12
    START = 0x13
    PARAM_QUERY = 0x40
    PARAM_SET = 0x80

    #Variables
    COUNT0 = 0x40
    COUNT1 = 0x80
    BITS_16 = 0x40
    BITS_24 = 0x00

    #I2C Config
    bus = SMBus(1)

    #Interrupt Pin
    si1133_int = 6

    #I2C Communication

    def _write_register(self, reg_addr, data):
        """Writes data to the I2C device.

        Parameters:
        reg_addr (byte): Write register address
        data (byte): Data to be written to the device"""

        self.bus.write_i2c_block_data(self.I2C_ADDRESS, reg_addr, [ data & 0xFF ])

    def _read_register_1ubyte(self, reg_addr):
        """Reads data from the I2C device.

        Parameters:
        reg_addr (byte): Read register address

        Returns:
        byte: Response from the device"""

        buffer = self.bus.read_i2c_block_data(self.I2C_ADDRESS, reg_addr, 1)
        return buffer[0]

    def _read_2bytes_lsbf(self, reg_addr):
        """Reads data from the I2C device.

        Parameters:
        reg_addr (byte): Read register address

        Returns:
        int: Response from the device, LSB first"""

        buffer = self.bus.read_i2c_block_data(self.I2C_ADDRESS, reg_addr, 2)
        return buffer[0] + (buffer[1] << 8)

    def _read_array(self, reg_addr, cnt):
        """Reads data from the I2C device.

        Parameters:
        reg_addr (byte): Read register start address
        cnt (byte): Read register address

        Returns:
        byte array: Response from the device"""

        return self.bus.read_i2c_block_data(self.I2C_ADDRESS, reg_addr, cnt)

    def _write_parameter(self, parameter, value):
        """Writes value to the parameter register.

        Parameters:
        parameter (byte): Parameter to be updated
        value (byte): Value of the parameter
        
        Returns:
        byte: Response from the device"""

        self._write_register(self.HOSTIN0, value)
        self._write_register(self.COMMAND, (parameter | self.PARAM_SET))
        return self._read_register_1ubyte(self.RESPONSE1)

    def _read_parameter(self, parameter):
        """Reads value from the parameter.

        Parameters:
        parameter (byte): Parameter to be read

        Returns:
        byte: Value of the parameter"""

        self._write_register(self.COMMAND, (parameter | self.PARAM_QUERY))
        return self._read_register_1ubyte(self.RESPONSE1)

    #Initialization

    def __init__(self):
        """Initiates the SI1133 sensor to get light and UV Index data."""

        GPIO.setwarnings(False)
        GPIO.setmode(GPIO.BCM)
        GPIO.setup(self.si1133_int, GPIO.IN, pull_up_down = GPIO.PUD_UP)
        
        #Reset
        self._reset()
        
        self._set_initial_settings()

        self.is_initialized = True
        sleep(0.5)

    #Sensor Configuration

    def _reset(self):
        """Resets the sensor."""

        self._write_register(self.COMMAND, self.RESET_SW)
        sleep(0.01)
        return

    def _set_initial_settings(self):
        """Writes the initial settings to the sensor."""

        self._write_parameter(self.CHLIST, 0x01)
        self._write_parameter(self.ADCCONFIG0, 0x78)
        self._write_parameter(self.ADCSENS0, 0x09)
        self._write_parameter(self.ADCPSOT0, 0x00)
        self._write_parameter(self.MEASCONFIG0, self.COUNT0)
        self._write_register(self.COMMAND, self.START)
        return

    #Sensor Read Methods

    def read_uv(self):
        """Reads the UV Index value.

        Returns:
        double: UV Index"""

        r1 = self._read_register_1ubyte(self.HOSTOUT0)
        r2 = self._read_register_1ubyte(self.HOSTOUT1)

        uvi = (r1 << 8) | r2

        if (uvi > 545):
            return 14
        else:
            return 0.0082 * (0.00391 * uvi * uvi + uvi)

    def read_ir(self):
        """Reads the IR value.

        Returns:
        int: Raw IR value"""

        r1 = self._read_register_1ubyte(self.HOSTOUT2)
        r2 = self._read_register_1ubyte(self.HOSTOUT3)
        r3 = self._read_register_1ubyte(self.HOSTOUT4)

        ir = (r1 << 16) | (r2 << 8) | r3
        return ir

    #Disposal

    def __del__(self):
        """Releases the resources."""

        try:
            if self.is_initialized:
                self._write_register(self.COMMAND, self.PAUSE)
                del self.is_initialized
        except:
            pass



# Turta Sensor uHAT Helper for Raspberry Pi OS.
# Distributed under the terms of the MIT license.

# Python Library for NXP MMA8491Q 3-Axis Accelerometer & Tilt Sensor.
# Version 1.0.0
# Released: June 14th, 2020

# Visit https://docs.turta.io for documentation.

import RPi.GPIO as GPIO
from smbus import SMBus
import time

class AccelTiltSensor(object):
    "MMA8491Q Accel & Tilt Sensor."

    #Variables
    is_initialized = False

    #Pins
    mma8491q_en, mma8491q_x_int, mma8491q_y_int, mma8491q_z_int = 25, 17, 27, 26

    #I2C Slave Address
    I2C_ADDRESS = 0x55

    #Registers
    MMA8491Q_STATUS = 0x00
    MMA8491Q_OUT_X_MSB = 0x01
    MMA8491Q_OUT_Y_MSB = 0x03
    MMA8491Q_OUT_Z_MSB = 0x05

    #I2C Config
    bus = SMBus(1)

    #I2C Communication

    def _read_register_1ubyte(self, reg_addr):
        """Reads data from the I2C device.

        Parameters:
        reg_addr (byte): Read register address

        Returns:
        byte: Response from the device"""

        buffer = self.bus.read_i2c_block_data(self.I2C_ADDRESS, reg_addr, 1)
        return buffer[0]

    def _read_2bytes_rs2(self, reg_addr):
        """Reads 2 bytes from the I2C device, and then shifts two bits right.

        Parameters:
        reg_addr (byte): Read register address

        Returns:
        int: Modified response from the device"""

        buffer = self.bus.read_i2c_block_data(self.I2C_ADDRESS, reg_addr, 2)
        return (buffer[0] << 6) + (buffer[1] >> 2)

    def _read_array(self, reg_addr, cnt):
        """Reads data from the I2C device.

        Parameters:
        reg_addr (byte): Read register start address
        cnt (byte): Read register address

        Returns:
        byte array: Response from the device"""

        return self.bus.read_i2c_block_data(self.I2C_ADDRESS, reg_addr, cnt)

    #Initialization

    def __init__(self):
        """Initiates the MMA8491Q sensor."""

        GPIO.setwarnings(False)
        GPIO.setmode(GPIO.BCM)
        GPIO.setup(self.mma8491q_en, GPIO.OUT)
        GPIO.setup(self.mma8491q_x_int, GPIO.IN, pull_up_down = GPIO.PUD_DOWN)
        GPIO.setup(self.mma8491q_y_int, GPIO.IN, pull_up_down = GPIO.PUD_DOWN)
        GPIO.setup(self.mma8491q_z_int, GPIO.IN, pull_up_down = GPIO.PUD_DOWN)
        self.is_initialized = True

    #Sensor Read Methods

    def _convert_to_g(self, analog_data):
        """Converts raw sensor data to G value.

        Parameters:
        analog_data (int): Raw sensor output data

        Returns:
        int: G Force of the given axis"""

        if ((analog_data & 0x2000) == 0x2000): #If zero or negative G
            return (0x3FFF - analog_data) / -1024.0
        else:                                  #If positive G
            return analog_data / 1024.0

    def read_accel_x(self):
        """Reads the G force of X-axis.

        Returns:
        int: G force of X-axis"""

        GPIO.output(self.mma8491q_en, GPIO.HIGH)
        time.sleep(0.001)

        while ((self._read_register_1ubyte(self.MMA8491Q_STATUS) & 0x01) != 0x01):
            time.sleep(0.001)
        tempData = self._read_2bytes_rs2(self.MMA8491Q_OUT_X_MSB)
        GPIO.output(self.mma8491q_en, GPIO.LOW)

        return self._convert_to_g(tempData)

    def read_accel_y(self):
        """Reads the G force of Y-axis.

        Returns:
        int: G force of Y-axis"""

        GPIO.output(self.mma8491q_en, GPIO.HIGH)
        time.sleep(0.001)

        while ((self._read_register_1ubyte(self.MMA8491Q_STATUS) & 0x02) != 0x02):
            time.sleep(0.001)
        tempData = self._read_2bytes_rs2(self.MMA8491Q_OUT_Y_MSB)
        GPIO.output(self.mma8491q_en, GPIO.LOW)

        return self._convert_to_g(tempData)

    def read_accel_z(self):
        """Reads the G force of Z-axis.

        Returns:
        int: G force of Z-axis"""

        GPIO.output(self.mma8491q_en, GPIO.HIGH)
        time.sleep(0.001)

        while ((self._read_register_1ubyte(self.MMA8491Q_STATUS) & 0x04) != 0x04):
            time.sleep(0.001)
        tempData = self._read_2bytes_rs2(self.MMA8491Q_OUT_Z_MSB)
        GPIO.output(self.mma8491q_en, GPIO.LOW)

        return self._convert_to_g(tempData)

    def read_accel_xyz(self):
        """Reads the G forces of X, Y and Z-axes.

        Returns:
        int array: G forces of X, Y and Z-axes respectively"""

        xyz = [ 0, 0, 0 ]

        GPIO.output(self.mma8491q_en, GPIO.HIGH)
        time.sleep(0.001)

        while ((self._read_register_1ubyte(self.MMA8491Q_STATUS) & 0x08) != 0x08):
            time.sleep(0.001)
        xyzArray = self._read_array(self.MMA8491Q_OUT_X_MSB, 6)
        GPIO.output(self.mma8491q_en, GPIO.LOW)

        xyz[0] = self._convert_to_g((xyzArray[0] << 6) + (xyzArray[1] >> 2)) #X-Axis
        xyz[1] = self._convert_to_g((xyzArray[2] << 6) + (xyzArray[3] >> 2)) #Y-Axis
        xyz[2] = self._convert_to_g((xyzArray[4] << 6) + (xyzArray[5] >> 2)) #Z-Axis

        return xyz

    def read_tilt_x(self):
        """Reads the X-Axis tilt state.

        Returns:
        bool: True if acceleration is > 0.688g or X-axis > 45, false if not"""

        GPIO.output(self.mma8491q_en, GPIO.HIGH)
        time.sleep(0.001)
        state = True if GPIO.input(self.mma8491q_x_int) else False
        GPIO.output(self.mma8491q_en, GPIO.LOW)

        return state

    def read_tilt_y(self):
        """Reads the Y-Axis tilt state.

        Returns:
        bool: True if acceleration is > 0.688g or Y-axis > 45, false if not"""

        GPIO.output(self.mma8491q_en, GPIO.HIGH)
        time.sleep(0.001)
        state = True if GPIO.input(self.mma8491q_y_int) else False
        GPIO.output(self.mma8491q_en, GPIO.LOW)

        return state

    def read_tilt_z(self):
        """Reads the Z-Axis tilt state.

        Returns:
        bool: True if acceleration is > 0.688g or Z-axis > 45, false if not"""

        GPIO.output(self.mma8491q_en, GPIO.HIGH)
        time.sleep(0.001)
        state = True if GPIO.input(self.mma8491q_z_int) else False
        GPIO.output(self.mma8491q_en, GPIO.LOW)

        return state

    def read_tilt_xyz(self):
        """Reads the X, Y and Z-Axis tilt states respectively.

        Returns:
        bool array: True if acceleration is > 0.688g or axis > 45, false if not"""

        xyz = [False, False, False]

        GPIO.output(self.mma8491q_en, GPIO.HIGH)
        time.sleep(0.001)
        xyz[0] = True if GPIO.input(self.mma8491q_x_int) else False
        xyz[1] = True if GPIO.input(self.mma8491q_y_int) else False
        xyz[2] = True if GPIO.input(self.mma8491q_z_int) else False
        GPIO.output(self.mma8491q_en, GPIO.LOW)

        return xyz

    #Disposal

    def __del__(self):
        """Releases the resources."""

        try:
            if self.is_initialized:
                GPIO.output(self.mma8491q_en, GPIO.LOW)
                GPIO.cleanup()
                del self.is_initialized
        except:
            pass



# Turta Sensor uHAT Helper for Raspberry Pi OS.
# Distributed under the terms of the MIT license.

# Python Library for IR Remote Controller Transmitter.
# Version 1.0.0
# Released: June 14th, 2020

# Visit https://docs.turta.io for documentation.

from enum import Enum
from time import sleep
import RPi.GPIO as GPIO

#Enumerations

#PROTOCOLS: IR Remote Controller Protocols
class PROTOCOLS(Enum):
    NEC = 0

class IRRemoteTx(object):
    """IR Remote Controller Transmitter."""

    #Variables
    is_initialized = False

    #IR LED Pin
    ir = 18

    #Initialize

    def __init__(self):
        GPIO.setwarnings(False)
        GPIO.setmode(GPIO.BCM)
        GPIO.setup(self.ir, GPIO.OUT, initial = GPIO.LOW)
        self.is_initialized = True
        return

    #Data Transmit

    def write(self, protocol, command):
        """Sends the IR remote controller signal using the selected protocol.

        Parameters:
        protocol (PROTOCOLS): Remote controller protocol
        command (byte array): Command to be sent"""

        if protocol not in PROTOCOLS:
            raise ValueError('protocol is not a member of PROTOCOLS.')

        if (protocol == PROTOCOLS.NEC):
            self._transmit_nec(command)

        return

    #IR Remote Controller Protocols

    #NEC Protocol
    def _transmit_nec(self, cmd):
        """Encodes and transmits the data in NEC protocol.

        Parameters:
        cmd (byte array): Command to be sent (2 Address + 2 Data = 4 Bytes)"""

        start_space_lenght_us_NEC = 0.0045 #4.5us
        burst_1_space_us = 0.001600 #0.001682s
        burst_0_space_us = 0.000534 #0.000534s

        #Leading Pulse
        self._nec_bit(320) #346

        #Delay
        sleep(start_space_lenght_us_NEC)

        #Data Bits
        for c in cmd: #4 Bytes
            for b in range(8): #8 Bits LSB First
                self._nec_bit(22)
                if (c & (1 << b)): #if 1
                    sleep(burst_1_space_us) #1.687mS space
                else: #if 0
                    sleep(burst_0_space_us) #562uS space

        #Stop Bit
        self._nec_bit(22)
        return

    def _nec_bit(self, cnt):
        """NEC Protocol pulse.

        Parameters:
        cnt (int): Number of pulses"""

        for x in range(cnt):
            GPIO.output(self.ir, GPIO.HIGH)
            GPIO.output(self.ir, GPIO.HIGH)
            GPIO.output(self.ir, GPIO.LOW)
            GPIO.output(self.ir, GPIO.LOW)
            GPIO.output(self.ir, GPIO.LOW)
            GPIO.output(self.ir, GPIO.LOW)
        return

    #Disposal

    def __del__(self):
        """Releases the resources."""

        try:
            if self.is_initialized:
                GPIO.output(self.ir, GPIO.LOW)
                GPIO.cleanup()
                del self.is_initialized
        except:
            pass



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="turta_sensoruhat",
    version="1.0.0",
    author="Turta LLC",
    author_email="hello@turta.io",
    description="Python Libraries for Turta Sensor uHAT",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://www.turta.io/sensoruhat",
    packages=setuptools.find_packages(),
    install_requires=[
        "RPi.GPIO",
        "smbus"
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
        "Topic :: System :: Hardware"
    ],
    project_urls={
        'Documentation': 'https://docs.turta.io/raspberry-pi-hats/sensor-uhat',
        'GitHub Repository' : 'https://github.com/Turta-io/Sensor-uHAT'
    },
)